# -*- coding: utf-8 -*-
import scrapy
import traceback
from .base import BaseSpider
from ..items import ExaItem


class TechcrunchSpider(BaseSpider):
    name = "tc"
    allowed_domains = ["techcrunch.com"]

    def start_requests(self):
        for i in self.companies(self.name):
            try:
                yield scrapy.Request(i.url + '/', callback=self.parse, meta={'company': i, 'post_id': 0})
            except:
                pass

    def parse(self, response):
        if 'tag' in response.url:
            return self.parse_tag(response)
        if 'company' in response.url:
            return self.parse_company(response)

    def parse_tag(self, response):
        try:
            news_list = response.xpath("..//div[contains(@class, 'block block-thumb ')]")
            company = response.meta['company']
            is_duplicate = False
            for i in news_list:
                item = ExaItem()
                item['date'] = i.xpath("./div/div/time/@datetime").extract_first()
                item['title'] = i.xpath("./div/h2/a/text()").extract_first()
                item['description'] = i.xpath("./div/p//text()").extract_first()
                item['url'] = i.xpath("./div/h2/a/@href").extract_first()

                item.update(self.get_common_items(company))

                item['post_id'] = response.meta['post_id']

                if self.pipeline.check_url(item['url']):
                    is_duplicate = True
                    break

                if item['title']:
                    yield scrapy.Request(item['url'], callback=self.parse_tags, meta={'item': item})

            has_next = response.xpath("//div[contains(@class, 'river-nav')]//li[contains(@class, 'next')]/a/@href").extract_first()
            if self.can_follow(has_next, is_duplicate):
                next_url = 'https://techcrunch.com' + has_next + '/'
                yield scrapy.Request(next_url, callback=self.parse_tag,
                                     meta=response.meta)
        except BaseException as e:
            print('We had error')
            traceback.print_exc()

    def parse_company(self, response):
        try:
            company = response.meta['company']
            news_list = response.xpath(".//div[contains(@class, 'block-content-topic')]")
            is_duplicate = False
            for i in news_list:
                item = ExaItem()
                item['date'] = i.xpath("./div/time/@datetime").extract_first()
                item['title'] = i.xpath("./h3/a/text()").extract_first()
                # Because we don't have description here
                item['description'] = None
                item['url'] = i.xpath("./h3/a/@href").extract_first()

                item.update(self.get_common_items(company))

                item['post_id'] = response.meta['post_id']

                if self.pipeline.check_url(item['url']):
                    is_duplicate = True
                    break
                if 'Crunch Report' not in item['title']:
                    yield scrapy.Request(item['url'], callback=self.parse_tags, meta={'item': item})

            has_next = response.xpath(
                "//div[contains(@class, 'river-nav')]//li[contains(@class, 'next')]/a/@href").extract_first()
            if self.can_follow(has_next, is_duplicate):
                yield scrapy.Request(has_next + '/', callback=self.parse, meta=response.meta)
            else:
                print("DUPLICATE NEWS")

        except BaseException as e:
            print('We had error')
            traceback.print_exc()

    def parse_tags(self, response):
        item = response.meta['item']
        item['tags'] = response.xpath(".//div[@class='accordion recirc-accordion']//ul//li[not(contains(@class, 'active'))]//a/text()").extract()

        yield item
